<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2019 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
defined('_JEXEC') or die();

if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
	return;
}

class PlgDPCalendarPayBraintree extends \DPCalendar\Plugin\PaymentPlugin
{
	protected function startTransaction($booking, $paymentProvider)
	{
		JLoader::import('dpcalendarpay.braintree.vendor.autoload', JPATH_PLUGINS);

		$gateway = new Braintree\Gateway([
			'environment' => $paymentProvider->sandbox == '1' ? 'sandbox' : 'production',
			'merchantId'  => trim($paymentProvider->api_merchant),
			'publicKey'   => trim($paymentProvider->api_public_key),
			'privateKey'  => trim($paymentProvider->api_private_key)
		]);

		// Generate the client token for the form
		$booking->client_token = $gateway->clientToken()->generate();

		return parent::startTransaction($booking, $paymentProvider);
	}

	protected function finishTransaction($data, $booking, $paymentProvider)
	{
		JLoader::import('dpcalendarpay.braintree.vendor.autoload', JPATH_PLUGINS);

		$gateway = new Braintree\Gateway([
			'environment' => $paymentProvider->sandbox == '1' ? 'sandbox' : 'production',
			'merchantId'  => trim($paymentProvider->api_merchant),
			'publicKey'   => trim($paymentProvider->api_public_key),
			'privateKey'  => trim($paymentProvider->api_private_key)
		]);

		// Get the transaction object
		$result = $gateway->transaction()->sale([
			'amount'             => $booking->price,
			'paymentMethodNonce' => $data['payment_nonce'],
			'options'            => ['submitForSettlement' => true]
		]);

		if ($result instanceof \Braintree\Result\Error) {
			throw new Exception($result->message);
		}

		$transaction = $result->transaction;

		// Collect the transaction details for the booking
		$updates = [
			'state'          => $transaction->status == 'submitted_for_settlement' ? 1 : 4,
			'payer_id'       => $transaction->customer['id'],
			'payer_email'    => $transaction->customer['email'],
			'transaction_id' => $transaction->id,
			'txn_type'       => $transaction->type,
			'txn_currency'   => $transaction->currencyIsoCode,
			'payment_fee'    => $transaction->transactionFeeAmount,
			'gross_amount'   => $transaction->amount,
			'net_amount'     => $transaction->amount - $transaction->transactionFeeAmount,
			'raw_data'       => json_encode($result)
		];

		return $updates;
	}
}
