<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2014 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
defined('_JEXEC') or die();

if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
	return;
}

use DigitalPeak\ThinHTTP as HTTP;
use Joomla\CMS\Date\Date;
use Joomla\Registry\Registry;

class PlgDPCalendarCalDAV extends \DPCalendar\Plugin\CalDAVPlugin
{
	protected $identifier = 'cad';

	/** @var \Joomla\CMS\Application\CMSApplication */
	protected $app;

	protected function getSyncToken($calendar)
	{
		$query = '<propfind xmlns="DAV:" xmlns:C="urn:ietf:params:xml:ns:caldav" xmlns:C1="http://calendarserver.org/ns/"><prop>
  <C1:getctag/>
 </prop></propfind>';

		$calendar->params->set('depth', 0);
		$response = $this->talkToDav(
			'/' . trim($calendar->params->get('calendar'), '/') . '/',
			$calendar->params,
			$query
		);
		$calendar->params->set('depth', null);

		if (!$response || empty($response->response) || empty($response->response->propstat) || empty($response->response->propstat->prop) || empty($response->response->propstat->prop->getctag)) {
			return parent::getSyncToken($calendar);
		}

		return $response->response->propstat->prop->getctag;
	}

	protected function createCalDAVEvent($uid, $icalData, $calendarId)
	{
		$calendar = $this->getDbCal($calendarId);
		if (empty($calendar)) {
			return;
		}

		$response = $this->talkToDav($calendar->params->get('calendar') . '/' . $uid, $calendar->params, $icalData, 'put');
		if ($response === null) {
			throw new Exception(JText::sprintf('JLIB_APPLICATION_ERROR_SAVE_FAILED', $uid));
		}
	}

	protected function updateCalDAVEvent($uid, $icalData, $calendarId)
	{
		$calendar = $this->getDbCal($calendarId);
		if (empty($calendar)) {
			return;
		}

		$path = $this->findEvent($uid, $calendar->params);
		if ($path === false) {
			$path = '/' . trim($calendar->params->get('calendar'), '/') . '/' . $uid . '.ics';
		}

		$response = $this->talkToDav($path, $calendar->params, $icalData, 'put');
		if ($response === null) {
			throw new Exception(JText::sprintf('JLIB_APPLICATION_ERROR_SAVE_FAILED', $uid));
		}
	}

	protected function deleteCalDAVEvent($uid, $calendarId)
	{
		$calendar = $this->getDbCal($calendarId);
		if (empty($calendar)) {
			return;
		}

		$path = $this->findEvent($uid, $calendar->params);
		if ($path === false) {
			$path = '/' . trim($calendar->params->get('calendar'), '/') . '/' . $uid . '.ics';
		}

		$response = $this->talkToDav($path, $calendar->params, null, 'delete', false);
		if ($response !== null) {
			return;
		}

		$path     = '/' . trim($calendar->params->get('calendar'), '/') . '/' . $uid;
		$response = $this->talkToDav($path, $calendar->params, null, 'delete', false);
		if ($response === null) {
			throw new Exception(JText::sprintf('JLIB_FILESYSTEM_DELETE_FAILED', $uid));
		}
	}

	protected function getOriginalData($uid, $calendarId)
	{
		$calendar = $this->getDbCal($calendarId);
		if (empty($calendar)) {
			return;
		}

		// On iCloud we need to fetch the event fully
		if (strpos($calendar->params->get('host'), 'icloud') !== false) {
			$eventData = $this->talkToDav(
				'/' . trim($calendar->params->get('calendar'), '/') . '/' . $uid . '.ics',
				$calendar->params,
				null,
				'get'
			);

			return trim(substr($eventData, strpos($eventData, 'BEGIN:VCALENDAR')));
		}

		$event = $this->findEvent($uid, $calendar->params, true);
		if (!$event) {
			return '';
		}

		if (empty($event->response->propstat) || empty($event->response->propstat->prop) || empty($event->response->propstat->prop->{'calendar-data'})) {
			return '';
		}

		return (string)$event->response->propstat->prop->{'calendar-data'};
	}

	protected function getContent($calendarId, Date $startDate = null, Date $endDate = null, Registry $options)
	{
		$calendar = $this->getDbCal($calendarId);
		if (empty($calendar)) {
			return '';
		}

		$query = '<C:calendar-query xmlns:D="DAV:" xmlns:C="urn:ietf:params:xml:ns:caldav"><D:prop><C:calendar-data/></D:prop>';
		$query .= '<C:filter><C:comp-filter name="VCALENDAR"><C:comp-filter name="VEVENT">';
		if ($startDate != null && $endDate != null) {
			$query .= '<C:time-range start="' . $startDate->format('Ymd\THis\Z') . '" end="' . $endDate->format('Ymd\THis\Z') . '"/>';
		}
		$query .= '</C:comp-filter></C:comp-filter></C:filter></C:calendar-query>';

		$data = $this->talkToDav('/' . trim($calendar->params->get('calendar'), '/') . '/', $calendar->params, $query, 'report');
		if (!$data) {
			return '';
		}

		$buffer = '';
		foreach ($data->response as $event) {
			$data = '';
			if (!empty($event->propstat) && !empty($event->propstat->prop) && !empty($event->propstat->prop->{'calendar-data'})) {
				$data = (string)$event->propstat->prop->{'calendar-data'};
			}
			if (empty($data) && !empty($event->href) && strpos($event->href, '.ics')) {
				$data = $this->talkToDav(
					'/' . trim($calendar->params->get('calendar'), '/') . '/' . $event['href'],
					$calendar->params,
					null,
					'get'
				);
			}

			if (!$data) {
				continue;
			}

			$start = strpos($data, 'BEGIN:VEVENT');
			$end   = strrpos($data, 'END:VEVENT');
			if ($start && $end) {
				$buffer .= substr($data, $start, $end - $start + 10) . PHP_EOL;
			}
		}

		$buffer = trim($buffer);
		$buffer = str_replace("BEGIN:VCALENDAR\r\n", '', $buffer);
		$buffer = str_replace("\r\nEND:VCALENDAR", '', $buffer);

		if (empty($buffer)) {
			return '';
		}

		return "BEGIN:VCALENDAR" . PHP_EOL . $buffer . PHP_EOL . "END:VCALENDAR";
	}

	public function import()
	{
		$params = new Registry($this->app->input->get('params', ['host' => null, 'username' => ''], 'array'));
		$params->set('host', trim($params['host'], '/') . '/');

		$response = '';
		if (strpos($params->get('host'), 'icloud') !== false) {
			$params->set('depth', 0);
			$response = $this->talkToDav(
				'',
				$params,
				"<propfind xmlns='DAV:'><prop><current-user-principal/></prop></propfind>"
			);
			$params->set('depth', null);
			if (empty($response)) {
				$this->app->redirect(
					JFactory::getSession()->get(
						'extcalendarOrigin',
						'index.php?option=com_dpcalendar&view=extcalendars&dpplugin=caldav&tmpl=' . $this->app->input->get('tmpl'),
						'DPCalendar'
					)
				);
				$this->app->close();
			}
			$userID = explode('/', $response->response[0]->propstat[0]->prop[0]->{'current-user-principal'}->href);
			$params->set('host', $params->get('host') . $userID[1]);
			$response = $this->talkToDav('/calendars/', $params, "<propfind xmlns='DAV:'><prop><displayname/></prop></propfind>");
		} else {
			$response = $this->talkToDav(
				'calendars/' . $params->get('username'),
				$params,
				"<A:propfind xmlns:A='DAV:'><A:prop><A:displayname/></A:prop></A:propfind>",
				'propfind',
				false
			);
			if (empty($response) || empty($response->response)) {
				$response = $this->talkToDav(
					'calendars/users/' . $params->get('username') . '/',
					$params,
					"<A:propfind xmlns:A='DAV:'><A:prop><A:displayname/></A:prop></A:propfind>",
					'propfind',
					false
				);
			}
			if (empty($response) || empty($response->response)) {
				$response = $this->talkToDav('', $params, "<A:propfind xmlns:A='DAV:'><A:prop><A:displayname/></A:prop></A:propfind>");
			}
		}

		if (empty($response) || empty($response->response)) {
			$this->app->redirect(
				JFactory::getSession()->get(
					'extcalendarOrigin',
					'index.php?option=com_dpcalendar&view=extcalendars&dpplugin=caldav&tmpl=' . $this->app->input->get('tmpl'),
					'DPCalendar'
				)
			);
			$this->app->close();
		}

		foreach ($response->response as $cal) {
			$model = JModelLegacy::getInstance('Extcalendar', 'DPCalendarModel');

			// Sometimes the calendar href contains the full path which is part of the host variable already
			$ref = (string)$cal->href;
			$pos = strpos($ref, 'calendars/');
			if ($pos !== false) {
				$ref = substr($ref, $pos);
			}
			$params->set('calendar', '/' . trim($ref, '/'));
			$params->set('action-create', true);
			$params->set('action-edit', true);
			$params->set('action-delete', true);

			$data          = [];
			$data['title'] = (string)$cal->propstat[0]->prop[0]->displayname;
			if (empty($data['title'])) {
				$data['title'] = $params->get('calendar');
			}
			$data['color']  = substr(dechex(crc32($data['title'])), 0, 6);
			$data['params'] = $params->toString();
			$data['plugin'] = 'caldav';

			if (!$model->save($data)) {
				$this->app->enqueueMessage($model->getError(), 'warning');
			}
		}

		$this->app->redirect(
			JFactory::getSession()->get(
				'extcalendarOrigin',
				'index.php?option=com_dpcalendar&view=extcalendars&dpplugin=caldav&tmpl=' . $this->app->input->get('tmpl'),
				'DPCalendar'
			)
		);
		$this->app->close();
	}

	private function findEvent($uid, $params, $fullResponse = false)
	{
		$query = '<C:calendar-query xmlns:D="DAV:" xmlns:C="urn:ietf:params:xml:ns:caldav">
  <D:prop><C:calendar-data/></D:prop>
  <C:filter><C:comp-filter name="VCALENDAR"><C:comp-filter name="VEVENT">
                <C:prop-filter name="UID"><C:text-match icollation="i;octet">' . $uid . '</C:text-match></C:prop-filter>
          </C:comp-filter></C:comp-filter></C:filter>
</C:calendar-query>';
		$event = $this->talkToDav('/' . trim($params->get('calendar'), '/'), $params, $query, 'report', false);
		if (empty($event->response) || empty($event->response->href)) {
			return false;
		}

		if ($fullResponse) {
			return $event;
		}

		return substr($event->response->href, strpos($event->response->href, $params->get('calendar')));
	}

	private function talkToDav($url, Registry $params, $query = '', $method = 'propfind', $log = true, $auth = CURLAUTH_BASIC)
	{
		try {
			$response = (new HTTP())->request(
				trim($params->get('host')) . trim($url),
				$query,
				$params->get('username'),
				$params->get('password'),
				['Depth: ' . $params->get('depth', 1), 'Content-Type: text/' . (strpos($query, 'BEGIN:') === 0 ? 'calendar' : 'xml')],
				[CURLOPT_HTTPAUTH => $auth],
				$method
			);

			if ($response->dp->info->http_code > 400) {
				throw new Exception('Failed connection with ' . $response->dp->info->http_code . ' error for url ' . $url . '!');
			}

			if ((strpos($response->dp->body, 'No digest authentication headers were found') !== false
				|| strpos($response->dp->body, 'No \'Authorization: Digest\' header found.') !== false)) {
				return $this->talkToDav($url, $params, $query, $method, CURLAUTH_DIGEST);
			}

			if ($response->dp->info->http_code < 200 || $response->dp->info->http_code > 299) {
				throw new Exception(!empty($response->dp->body) ? $response->dp->body : 'Error fetching CalDAV data!');
			}

			if (strpos($response->dp->body, '<') !== 0) {
				return $response->dp->body;
			}

			// Remove the namespaces
			$body = preg_replace('/xmlns[^=]*="[^"]*"/i', '', $response->dp->body);
			$body = str_ireplace('<d:', '<', $body);
			$body = str_ireplace('</d:', '</', $body);
			$body = str_ireplace('<c:', '<', $body);
			$body = str_ireplace('</c:', '</', $body);
			$body = str_ireplace('<s:', '<', $body);
			$body = str_ireplace('</s:', '</', $body);
			$body = str_ireplace('<cal:', '<', $body);
			$body = str_ireplace('</cal:', '</', $body);

			$response = new SimpleXMLElement($body);
			if ($response->exception) {
				throw new Exception($response->message);
			}

			return $response;
		} catch (Exception $e) {
			if ($log) {
				$this->log($e->getMessage());
			}

			return null;
		}
	}
}
