<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2021 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */

defined('_JEXEC') or die();

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Router\ApiRouter;
use Joomla\CMS\Serializer\Events\OnGetApiAttributes;
use Joomla\Router\Route;

class PlgWebservicesDPCalendar extends CMSPlugin
{
	protected $autoloadLanguage = true;

	public function onBeforeApiRoute(ApiRouter &$router)
	{
		$router->createCRUDRoutes(
			'v1/dpcalendar/events',
			'events',
			['component' => 'com_dpcalendar']
		);
		$router->createCRUDRoutes(
			'v1/dpcalendar/locations',
			'locations',
			['component' => 'com_dpcalendar']
		);

		$router->createCRUDRoutes(
			'v1/dpcalendar/calendars',
			'categories',
			['component' => 'com_categories', 'extension' => 'com_dpcalendar']
		);

		$this->createFieldsRoutes($router);

		$this->createContentHistoryRoutes($router);
	}

	public function onGetApiAttributes(OnGetApiAttributes $event)
	{
		if ($event->getArgument('context') === 'events') {
			$event->addAttribute('calid', $event->getArguments()['attributes']['catid']);
		}
	}

	private function createFieldsRoutes(&$router)
	{
		$router->createCRUDRoutes(
			'v1/fields/dpcalendar/events',
			'fields',
			['component' => 'com_fields', 'context' => 'com_dpcalendar.event']
		);

		$router->createCRUDRoutes(
			'v1/fields/dpcalendar/locations',
			'fields',
			['component' => 'com_fields', 'context' => 'com_dpcalendar.location']
		);

		$router->createCRUDRoutes(
			'v1/fields/dpcalendar/calendars',
			'fields',
			['component' => 'com_fields', 'context' => 'com_dpcalendar.categories']
		);

		$router->createCRUDRoutes(
			'v1/fields/groups/dpcalendar/events',
			'groups',
			['component' => 'com_fields', 'context' => 'com_dpcalendar.event']
		);

		$router->createCRUDRoutes(
			'v1/fields/groups/dpcalendar/locations',
			'groups',
			['component' => 'com_fields', 'context' => 'com_dpcalendar.location']
		);

		$router->createCRUDRoutes(
			'v1/fields/groups/dpcalendar/calendars',
			'groups',
			['component' => 'com_fields', 'context' => 'com_dpcalendar.categories']
		);
	}

	private function createContentHistoryRoutes(&$router)
	{
		$defaults = [
			'component'  => 'com_contenthistory',
			'type_alias' => 'com_dpcalendar.event',
			'type_id'    => 2,
		];
		$getDefaults = array_merge(['public' => false], $defaults);

		$routes = [
			new Route(['GET'], 'v1/dpcalendar/:id/contenthistory', 'history.displayList', ['id' => '(\d+)'], $getDefaults),
			new Route(['PATCH'], 'v1/dpcalendar/:id/contenthistory/keep', 'history.keep', ['id' => '(\d+)'], $defaults),
			new Route(['DELETE'], 'v1/dpcalendar/:id/contenthistory', 'history.delete', ['id' => '(\d+)'], $defaults),
		];

		$router->addRoutes($routes);
	}
}
