<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2022 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\PluginHelper;

require_once JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/fields/checkboxgroup.php';

class RSFormProFieldDPCalendarReservation extends RSFormProFieldCheckboxGroup
{
	public function getPreviewInput()
	{
		return $this->loadCalendar(parent::getPreviewInput());
	}

	public function getFormInput()
	{
		return $this->loadCalendar(parent::getFormInput());
	}

	public function getAttributes()
	{
		$attributes = parent::getAttributes();

		if (!array_key_exists('class', $attributes)) {
			$attributes['class'] = '';
		}

		$attributes['class'] .= ' dp-time-options__option';

		return $attributes;
	}

	private function loadCalendar(string $optionsCode)
	{
		if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
			return $optionsCode;
		}

		$app = Factory::getApplication();

		$app->getLanguage()->load('com_dpcalendar', JPATH_ADMINISTRATOR . '/components/com_dpcalendar');
		$app->getLanguage()->load('com_dpcalendar', JPATH_SITE . '/components/com_dpcalendar');

		// Variables for the scripts
		$calendarId = $this->data['CALENDAR_ID'];
		$id         = $this->componentId;
		$name       = $this->name;

		$params = clone ComponentHelper::getParams('com_dpcalendar');
		$params->set('calendar_height', $this->data['CALENDAR_HEIGHT'] ?: 0);

		$document     = new \DPCalendar\HTML\Document\HtmlDocument();
		$dateHelper   = new \DPCalendar\Helper\DateHelper();
		$layoutHelper = new \DPCalendar\Helper\LayoutHelper();
		$userHelper   = new \DPCalendar\Helper\UserHelper();
		$router       = new \DPCalendar\Router\Router();
		$translator   = new \DPCalendar\Translator\Translator();

		// The display data
		$displayData = [
			'document'     => $document,
			'layoutHelper' => $layoutHelper,
			'userHelper'   => $userHelper,
			'dateHelper'   => $dateHelper,
			'translator'   => $translator,
			'router'       => $router,
			'params'       => $params
		];

		ob_start();
		include PluginHelper::getLayoutPath('system', 'dpcalendarrsform', 'default');
		return ob_get_clean();
	}
}
