<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2020 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
defined('_JEXEC') or die();

if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
	return;
}

use Firebase\JWT\JWT;
use Joomla\CMS\Date\Date;
use Joomla\Registry\Registry;

class PlgDPCalendarJitsi extends \DPCalendar\Plugin\SyncPlugin
{
	protected $identifier = 'ji';

	/** @var \Joomla\CMS\Application\CMSApplication */
	protected $app;

	public function onContentAfterDisplay($context, $event)
	{
		if ($context != 'com_dpcalendar.event') {
			return;
		}

		$params = $event->params;
		if (!$params->get('jitsi_room') || !$params->get('jitsi_calendar')) {
			return;
		}

		$user = JFactory::getUser();
		if ($params->get('jitsi_only_tickets') && $user->guest) {
			return;
		}

		if ($params->get('jitsi_only_tickets') && $event->created_by != JFactory::getUser()->id) {
			$myTickets = array_filter($event->tickets, function ($ticket) use ($user) {
				return $ticket->user_id == $user->id && $ticket->state == 1;
			});

			if (!$myTickets) {
				return;
			}
		}

		$calendar = $this->getDbCal($params->get('jitsi_calendar'));
		if (!$calendar) {
			return;
		}

		// When the event has not started yet load the join template
		if (\DPCalendar\Helper\DPCalendarHelper::getDate()->toSql() >= $event->end_date) {
			return;
		}

		$path       = JPluginHelper::getLayoutPath('dpcalendar', 'jitsi', 'join');
		$document   = new \DPCalendar\HTML\Document\HtmlDocument();
		$translator = new DPCalendar\Translator\Translator();

		ob_start();
		include $path;

		return ob_get_clean();
	}

	public function onContentPrepareForm($form, $data)
	{
		if ($form->getName() != 'com_dpcalendar.event') {
			return true;
		}

		$obj = (object)$data;
		// On save data is empty so add the form
		if (!empty($obj->id) && !is_numeric($obj->id)) {
			return true;
		}

		$form->loadFile(JPATH_PLUGINS . '/dpcalendar/jitsi/forms/event.xml');

		parent::onContentPrepareForm($form, $data);
	}
}
