<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2014 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */

defined('_JEXEC') or die();

use DPCalendar\Booking\Stages\CollectEventsAndTickets;
use DPCalendar\Booking\Stages\SetupForMail;
use DPCalendar\Helper\Booking;
use DPCalendar\Helper\DateHelper;
use DPCalendar\Helper\DPCalendarHelper;
use DPCalendar\Plugin\PaymentPlugin;
use DPCalendar\Translator\Translator;
use DPCalendarPay\Manual\SendInvoice;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\CMS\Plugin\PluginHelper;
use League\Pipeline\PipelineBuilder;

if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
	return;
}

class PlgDPCalendarPayManual extends PaymentPlugin
{
	protected function getInvoiceData($booking, $paymentProvider): array
	{
		$variables                   = [];
		$variables['paymentDueDate'] = DPCalendarHelper::getDate($booking->book_date, true)->modify('+30 days')
			->format(DPCalendarHelper::getComponentParameter('event_date_format', 'd.m.Y'));
		$variables['bankInformation'] = $this->getPaymentStatement($paymentProvider, $booking, $variables['paymentDueDate']);

		$tickets    = $booking->tickets;
		$translator = new Translator();
		$dateHelper = new DateHelper();
		$params     = ComponentHelper::getParams('com_dpcalendar');

		ob_start();
		require PluginHelper::getLayoutPath('dpcalendarpay', 'manual', 'invoice');

		$fileName = Booking::createPDF(ob_get_clean(), $booking->uid, true);
		if (file_exists($fileName)) {
			return ['name' => $booking->uid . '.pdf', 'content' => file_get_contents($fileName)];
		}

		return parent::getInvoiceData($booking, $paymentProvider);
	}

	protected function finishTransaction($data, $booking, $paymentProvider)
	{
		// Only send out a mail when state is confirmed, can happen for waiting lists
		if ($booking->state == 3 && $this->app->get('mailonline', false)) {
			require_once __DIR__ . '/src/SendInvoice.php';

			$builder = new PipelineBuilder();
			$builder->add(new CollectEventsAndTickets(
				Factory::getApplication(),
				BaseDatabaseModel::getInstance('Booking', 'DPCalendarModel', ['ignore_request' => true])
			));
			$builder->add(new SetupForMail(Factory::getApplication(), ComponentHelper::getParams('com_dpcalendar')));
			$builder->add(new SendInvoice(Factory::getMailer()));

			$builder->build()->process(
				(object)[
					'oldItem'         => $booking,
					'item'            => $booking,
					'bankInformation' => $this->getPaymentStatement($paymentProvider, $booking, $variables['paymentDueDate']),
					'language'        => $this->app->getLanguage()
				]
			);
		}

		// Set state to needs payment and some other fields
		$updates = [
			'state'          => !empty($paymentProvider->state) ? $paymentProvider->state : 4,
			'payer_email'    => $booking->email,
			'transaction_id' => md5($booking->id . 'dpcalendar'),
			'invoice'        => 1
		];

		return $updates;
	}

	private function getPaymentStatement($provider, $booking, $date)
	{
		$vars                               = (array)$booking;
		$vars['paymentDueDate']             = $date;
		$vars['currency']                   = DPCalendarHelper::getComponentParameter('currency', 'USD');
		$vars['currencySymbol']             = DPCalendarHelper::getComponentParameter('currency_symbol', '$');
		$vars['currencySeparator']          = DPCalendarHelper::getComponentParameter('currency_separator', '.');
		$vars['currencyThousandsSeparator'] = DPCalendarHelper::getComponentParameter('currency_thousands_separator', "'");
		$vars['price_formatted']            = DPCalendarHelper::renderPrice(
			$vars['price'],
			$vars['currencySymbol'],
			$vars['currencySeparator'],
			$vars['currencyThousandsSeparator']
		);

		if (!empty($booking->jcfields)) {
			foreach ($booking->jcfields as $field) {
				$vars['field-' . $field->name] = $field;
			}
		}

		$text = trim(strip_tags($provider->payment_statement));
		$text = $this->app->getLanguage()->hasKey($text) ? $this->app->getLanguage()->_($text) : $provider->payment_statement;

		return DPCalendarHelper::renderEvents([], $text, ComponentHelper::getParams('com_dpcalendar'), $vars);
	}
}
