/**
 * @package   DPCalendar
 * @copyright Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
(function () {
	'use strict';
	document.addEventListener('DOMContentLoaded', () => {
		[].slice.call(document.querySelectorAll('.plg-system-dpcalendarrsform__calendar')).forEach((calendar) => {
			calendar.addEventListener('dp-calendar-loaded', (event) => {
				const options = [].slice.call(calendar.parentElement.querySelectorAll('.dp-time-options__option'));
				options.forEach((option) => {
					option.disabled = true;
					option.addEventListener('click', (event) => {
						options.forEach((optionToCheck) => {
							if (optionToCheck.value === option.value) {
								return;
							}
							optionToCheck.checked = false;
						});
					});
				});
				event.detail.calendar.setOption('dateClick', (info) => {
					options.forEach((option) => {
						option.disabled = false;
						option.checked = false;
					});
					if (info.dayEl.classList.contains('plg-system-dpcalendarrsform__calendar_selected')) {
						info.dayEl.classList.remove('plg-system-dpcalendarrsform__calendar_selected');
						options.forEach((option) => option.disabled = true);
						return;
					}
					[].slice.call(document.querySelectorAll('.plg-system-dpcalendarrsform__calendar_selected')).forEach((selectedCell) => {
						selectedCell.classList.remove('plg-system-dpcalendarrsform__calendar_selected');
					});
					info.dayEl.classList.add('plg-system-dpcalendarrsform__calendar_selected');
					calendar.parentElement.querySelector('.dp-time-options__date').value = info.dateStr;
					options.forEach((option) => option.disabled = false);
					event.detail.calendar.getEvents().forEach((event) => {
						if (info.dateStr != event.startStr) {
							return;
						}
						options.forEach((option) => {
							const overlappingEvent = event.extendedProps.eventData.find((eventOnDay) => {
								let eventStart = dayjs(eventOnDay.start_date);
								let eventEnd = dayjs(eventOnDay.end_date);
								if (eventOnDay.all_day == 1) {
									eventEnd = eventEnd.add(1, 'day');
								}
								let optionStart;
								let optionEnd;
								[optionStart, optionEnd] = option.value.split('-');
								optionStart = dayjs(event.startStr + ' ' + optionStart);
								optionEnd = dayjs(event.startStr + ' ' + optionEnd);
								return eventStart <= optionEnd && eventEnd >= optionStart;
							});
							if (overlappingEvent) {
								option.disabled = true;
							}
						});
					});
				});
			});
		});
	});
})();
