/**
 * @package   DPCalendar
 * @copyright Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
(function () {
	'use strict';
	document.addEventListener('DOMContentLoaded', () => {
		const loader = document.querySelector('.dp-loader');
		if (loader) {
			loader.classList.remove('dp-loader_hidden');
		}
		loadDPAssets(['https://js.braintreegateway.com/web/3.68.0/js/client.min.js', 'https://js.braintreegateway.com/web/3.68.0/js/three-d-secure.min.js', 'https://js.braintreegateway.com/web/3.68.0/js/hosted-fields.min.js', '/com_dpcalendar/js/dpcalendar/dpcalendar.js'], () => {
			const root = document.querySelector('.plg-dpcalendarpay-braintree');
			const form = root.querySelector('.plg-dpcalendarpay_braintree__form');
			const submit = root.querySelector('.dp-button-submit');
			const messageBox = root.querySelector('.plg-dpcalendarpay-braintree__message');
			if (loader) {
				loader.classList.add('dp-loader_hidden');
			}
			braintree.client.create(
				{authorization: root.getAttribute('data-key')},
				(clientErr, clientInstance) => {
					if (clientErr) {
						messageBox.textContent = clientErr.message;
						messageBox.style.display = 'block';
						return;
					}
					braintree.hostedFields.create(
						{
							client: clientInstance,
							styles: {
								'input': '.dp-input'
							},
							fields: {
								number: {
									selector: '.dp-card__number'
								},
								cvv: {
									selector: '.dp-card__cvc'
								},
								expirationDate: {
									selector: '.dp-card__expiry'
								}
							}
						},
						(hostedFieldsErr, hostedFieldsInstance) => {
							if (hostedFieldsErr) {
								messageBox.textContent = hostedFieldsErr.message;
								messageBox.style.display = 'block';
								return;
							}
							let threeDSecure;
							braintree.threeDSecure.create({
								authorization: root.getAttribute('data-key'),
								version: 2
							}, (threeDSecureErr, threeDSecureInstance) => {
								if (threeDSecureErr) {
									messageBox.textContent = threeDSecureErr.message;
									messageBox.style.display = 'block';
									return;
								}
								threeDSecure = threeDSecureInstance;
							});
							submit.disabled = false;
							submit.addEventListener(
								'click',
								(event) => {
									event.preventDefault();
									submit.disabled = true;
									[].slice.call(root.querySelectorAll('.dp-card-control')).forEach((group) => {
										group.classList.remove('dp-card-control_error');
									});
									messageBox.style.display = 'none';
									if (loader) {
										loader.classList.remove('dp-loader_hidden');
									}
									hostedFieldsInstance.tokenize((error, payload) => {
										if (!error) {
											if (!threeDSecure) {
												submit.disabled = false;
												if (loader) {
													loader.classList.add('dp-loader_hidden');
												}
												return;
											}
											threeDSecure.verifyCard({
												nonce: payload.nonce, amount: root.getAttribute('data-amount'), onLookupComplete: (data, next) => {
													next();
												}
											}).then((payload) => {
												if (!payload.liabilityShifted) {
													messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_BRAINTREE_FORM_LIABILITY_NOT_SHIFTED');
													messageBox.style.display = 'block';
													submit.disabled = false;
													if (loader) {
														loader.classList.add('dp-loader_hidden');
													}
													return;
												}
												root.querySelector('.dp-form-input[name="payment_nonce"]').value = payload.nonce;
												form.submit();
											});
											return;
										}
										submit.disabled = false;
										if (loader) {
											loader.classList.add('dp-loader_hidden');
										}
										if (error.details && error.details.invalidFields) {
											if (error.details.invalidFields.number) {
												error.details.invalidFields.number.closest('.dp-card-control').classList.add('dp-card-control_error');
											}
											if (error.details.invalidFields.cvv) {
												error.details.invalidFields.cvv.closest('.dp-card-control').classList.add('dp-card-control_error');
											}
											if (error.details.invalidFields.expirationDate) {
												error.details.invalidFields.expirationDate.closest('.dp-card-control').classList.add('dp-card-control_error');
											}
										}
										let message = error.message;
										switch (error.code) {
											case 'HOSTED_FIELDS_FIELDS_EMPTY':
												message = Joomla.JText._('PLG_DPCALENDARPAY_BRAINTREE_FORM_EMPTY');
												break;
											case 'HOSTED_FIELDS_FIELDS_INVALID':
												message = Joomla.JText._('PLG_DPCALENDARPAY_BRAINTREE_FORM_INVALID');
												break;
											case 'HOSTED_FIELDS_TOKENIZATION_FAIL_ON_DUPLICATE':
												message = Joomla.JText._('PLG_DPCALENDARPAY_BRAINTREE_FORM_FAIL_ON_DUPLICATE');
												break;
											case 'HOSTED_FIELDS_TOKENIZATION_CVV_VERIFICATION_FAILED':
												message = Joomla.JText._('PLG_DPCALENDARPAY_BRAINTREE_FORM_INVALID_CVC');
												break;
											case 'HOSTED_FIELDS_FAILED_TOKENIZATION':
												message = Joomla.JText._('PLG_DPCALENDARPAY_BRAINTREE_FORM_TOKENIZATION');
												break;
											case 'HOSTED_FIELDS_TOKENIZATION_NETWORK_ERROR':
												message = Joomla.JText._('PLG_DPCALENDARPAY_BRAINTREE_FORM_NETWORK');
										}
										messageBox.textContent = message;
										messageBox.style.display = 'block';
									});
								},
								false
							);
						}
					);
				}
			);
		});
	});
})();
