/**
 * @package   DPCalendar
 * @copyright Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
(function () {
	'use strict';
	document.addEventListener('DOMContentLoaded', () => {
		loadDPAssets(['https://js.stripe.com/v3', '/com_dpcalendar/js/dpcalendar/dpcalendar.js'], () => {
			const root = document.querySelector('.plg-dpcalendarpay-stripe');
			const stripe = Stripe(root.getAttribute('data-key'));
			const elements = stripe.elements({locale: document.getElementsByTagName('html')[0].getAttribute('lang')});
			elements.create('cardNumber').mount('.dp-card__number');
			elements.create('cardExpiry').mount('.dp-card__expiry');
			elements.create('cardCvc').mount('.dp-card__cvc');
			const cardButton = root.querySelector('.dp-button-submit');
			cardButton.addEventListener('click', (ev) => {
				ev.preventDefault();
				cardButton.disabled = true;
				stripe.confirmCardPayment(root.getAttribute('data-secret'), {
					payment_method: {
						card: elements.getElement('cardNumber'),
						billing_details: {name: root.querySelector('.dp-card__holder').value}
					}
				}).then((result) => {
					[].slice.call(root.querySelectorAll('.dp-card-control')).forEach((group) => group.classList.remove('dp-card-control_error'));
					cardButton.disabled = false;
					const messageBox = root.querySelector('.plg-dpcalendarpay-stripe__message');
					if (!result.error) {
						messageBox.display = 'none';
						root.querySelector('.dp-form-input[name="intent"]').value = result.paymentIntent.id;
						root.querySelector('.dp-form').submit();
						return;
					}
					switch (result.error.code) {
						case 'incorrect_number':
						case 'incomplete_number':
							root.querySelector('.dp-card__number').closest('.dp-card-control').classList.add('dp-card-control_error');
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_INCORRECT_NUMBER');
							break;
						case 'invalid_number':
							root.querySelector('.dp-card__number').closest('.dp-card-control').classList.add('dp-card-control_error');
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_INVALID_NUMBER');
							break;
						case 'invalid_expiry_month':
							root.querySelector('.dp-card__expiry').closest('.dp-card-control').classList.add('dp-card-control_error');
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_INVALID_EXP_MONTH');
							break;
						case 'invalid_expiry_year':
						case 'invalid_expiry_year_past':
							root.querySelector('.dp-card__expiry').closest('.dp-card-control').classList.add('dp-card-control_error');
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_INVALID_EXP_YEAR');
							break;
						case 'invalid_cvc':
						case 'incomplete_cvc':
							root.querySelector('.dp-card__cvc').closest('.dp-card-control').classList.add('dp-card-control_error');
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_INVALID_CVC');
							break;
						case 'expired_card':
							root.querySelector('.dp-card__expiry').closest('.dp-card-control').classList.add('dp-card-control_error');
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_EXPIRED_CARD');
							break;
						case 'incorrect_cvc':
							root.querySelector('.dp-card__cvc').closest('.dp-card-control').classList.add('dp-card-control_error');
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_INCORRECT_CVC');
							break;
						case 'card_declined':
							root.querySelector('.dp-card__number').closest('.dp-card-control').classList.add('dp-card-control_error');
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_CARD_DECLINED');
							break;
						case 'missing':
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_MISSING');
							break;
						case 'processing_error':
							messageBox.textContent = Joomla.JText._('PLG_DPCALENDARPAY_STRIPE_FORM_PROCESSING_ERROR');
							break;
						default:
							messageBox.textContent = result.error.message;
					}
					messageBox.style.display = 'block';
				});
				return false;
			});
		});
	});
})();
