<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2019 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
defined('_JEXEC') or die();

if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
	return;
}

class PlgDPCalendarPayMollie extends \DPCalendar\Plugin\PaymentPlugin
{
	protected function startTransaction($booking, $paymentProvider)
	{
		$params = $this->getPurchaseParameters($booking);

		$data = [
			'amount'      => ['value' => $params['amount'], 'currency' => $params['currency']],
			'description' => $params['description'],
			'redirectUrl' => $params['returnUrl']
		];

		$response = $this->getHTTP()->post(
			'https://api.mollie.com/v2/payments',
			json_encode($data),
			trim($paymentProvider->api_key)
		);

		if ($response->dp->info->http_code >= 400) {
			throw new Exception(!empty($response->title) ? $response->title : 'Could not create Mollie transaction!');
		}

		if ($response->status != 'open') {
			throw new Exception('Could not create Mollie transaction!');
		}

		// Store the transaction id in the booking, is needed on the callback
		if (!empty($response->id)) {
			// Get the booking table
			\JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_dpcalendar/tables');
			$bookingTable = \JTable::getInstance('Booking', 'DPCalendarTable');

			// Load the booking so we can store the transaction id
			$bookingTable->load($booking->id);
			$bookingTable->transaction_id = $response->id;
			$bookingTable->store();
		}

		$this->app->redirect($response->_links->checkout->href);
	}

	protected function finishTransaction($data, $booking, $paymentProvider)
	{
		$response = $this->getHTTP()->get(
			'https://api.mollie.com/v2/payments/' . $booking->transaction_id,
			trim($paymentProvider->api_key)
		);

		if ($response->dp->info->http_code >= 400) {
			throw new Exception(!empty($response->title) ? $response->title : 'Could not fetch Mollie transaction!');
		}

		// Set some values from the transaction Mollie is very limited here
		$updates = [
			'state'          => 0,
			'payer_id'       => $response->profileId,
			'transaction_id' => $response->id,
			'txn_type'       => $response->method,
			'txn_currency'   => $response->amount->currency,
			'gross_amount'   => $response->amount->value,
			// Mollie doesn't have the net amount or fee available yet
			// https://github.com/mollie/mollie-api-php/issues/296
			'net_amount' => $response->amount->value,
			'raw_data'   => json_encode($response)
		];

		// Determine the state
		switch ($response->status) {
			case 'paid':
				$updates['state'] = 1;
				break;
			case 'open':
			case 'pending':
				$updates['state'] = 4;
				break;
			case 'failed':
			case 'canceled':
			case 'expired':
				$updates['state'] = 6;
				break;
		}

		return $updates;
	}

	protected function getPurchaseParameters($booking)
	{
		// Collect the base purchase parameters
		$purchaseParameters                = parent::getPurchaseParameters($booking);
		$purchaseParameters['description'] = '[' . $booking->id . ']';

		// Set the description based on the tickets
		if ($booking->tickets) {
			$ticket                            = reset($booking->tickets);
			$purchaseParameters['description'] = $purchaseParameters['description'] . ' ' . $ticket->event_title;
		}

		return $purchaseParameters;
	}
}
