<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2020 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
defined('_JEXEC') or die();

if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
	return;
}

use Joomla\CMS\Date\Date;
use Joomla\Registry\Registry;

class PlgDPCalendarPrivate extends \DPCalendar\Plugin\CalDAVPlugin
{
	protected $identifier = 'cd';

	protected function createCalDAVEvent($uid, $icalData, $calendarId)
	{
		return $this->getBackend()->createCalendarObject([$calendarId], $uid . '.ics', $icalData);
	}

	protected function updateCalDAVEvent($uid, $icalData, $calendarId)
	{
		return $this->getBackend()->updateCalendarObject([$calendarId], $uid . '.ics', $icalData);
	}

	protected function deleteCalDAVEvent($uid, $calendarId)
	{
		return $this->getBackend()->deleteCalendarObject([$calendarId], $uid . '.ics');
	}

	protected function getOriginalData($uid, $calendarId)
	{
		$db = JFactory::getDbo();
		$db->setQuery("select * from #__dpcalendar_caldav_calendarobjects where uri = " . $db->quote($db->escape($uid) . '.ics'));
		$row = $db->loadObject();

		return $row->calendardata;
	}

	protected function getContent($calendarId, Date $startDate = null, Date $endDate = null, Registry $options)
	{
		$db   = JFactory::getDbo();
		$user = JFactory::getUser();

		$join = $db->getQuery(true);
		$join->select("distinct REPLACE( REPLACE( linkp.uri,  '/calendar-proxy-write',  '' ) ,  '/calendar-proxy-read',  '' )");
		$join->from($db->quoteName('#__dpcalendar_caldav_principals') . ' memberp');
		$join->join('inner', "#__dpcalendar_caldav_groupmembers m ON memberp.id = m.member_id");
		$join->join('inner', "#__dpcalendar_caldav_principals linkp ON m.principal_id = linkp.id");
		$join->where('memberp.external_id = ' . (int)$user->id);

		$query = $db->getQuery(true);

		$query->select('e.*');
		$query->from($db->quoteName('#__dpcalendar_caldav_calendarobjects') . ' AS e');
		$query->join('RIGHT', '#__dpcalendar_caldav_calendarinstances c on c.id = e.calendarid');
		$query->where('(c.principaluri = ' . $db->quote('principals/' . $user->username) . ' or c.principaluri in (' . $join . '))');
		$query->where('c.id = ' . (int)$calendarId);

		if ($startDate != null) {
			$start         = $startDate->format('U');
			$dateCondition = 'e.firstoccurence  >= ' . $start;
			if ($endDate !== null) {
				$end           = $endDate->format('U');
				$dateCondition = '(e.lastoccurence between ' . $start . ' and ' . $end . ' or e.firstoccurence between ' . $start . ' and ' . $end .
					')';
			}
		}

		$query->order('firstoccurence asc');

		$db->setQuery($query);
		$rows = $db->loadObjectList();
		if (empty($rows)) {
			return [];
		}

		$text   = [];
		$text[] = 'BEGIN:VCALENDAR';
		foreach ($rows as $item) {
			if (empty($item->calendardata)) {
				continue;
			}
			$start = strpos($item->calendardata, 'BEGIN:VEVENT');
			$end   = strrpos($item->calendardata, 'END:VEVENT');
			$tmp   = substr($item->calendardata, $start, $end - $start + 10);

			if (strpos($tmp, 'ORGANIZER:') === false) {
				$tmp = str_replace('END:VEVENT', 'ORGANIZER:' . $user->name . PHP_EOL . 'END:VEVENT', $tmp);
			}

			$text = array_merge($text, explode(PHP_EOL, $tmp));
		}

		$text[] = 'END:VCALENDAR';

		// Echo '<pre>';print_R($text);echo '</pre>';die;
		return $text;
	}

	public function fetchCalendars($calendarIds = null)
	{
		JModelLegacy::addIncludePath(JPATH_SITE . '/components/com_dpcalendar/models', 'DPCalendarModel');
		$model = JModelLegacy::getInstance('Profile', 'DPCalendarModel');

		$data  = [];
		$items = $model->getItems();

		if (empty($items)) {
			return $data;
		}

		foreach ($items as $caldavCalendar) {
			$calendar = $this->createCalendar(
				$caldavCalendar->id,
				$caldavCalendar->displayname,
				$caldavCalendar->description,
				$caldavCalendar->calendarcolor
			);

			if (!empty($calendarIds) && !in_array($caldavCalendar->id, $calendarIds)) {
				continue;
			}

			$calendar->params    = new Registry();
			$calendar->canCreate = $caldavCalendar->canEdit;
			$calendar->canEdit   = $caldavCalendar->canEdit;
			$calendar->canDelete = $caldavCalendar->canEdit;
			$data[]              = $calendar;
		}

		return $data;
	}

	private function getBackend()
	{
		$config = JFactory::getApplication();

		JLoader::import('components.com_dpcalendar.vendor.autoload', JPATH_ADMINISTRATOR);

		$pdo = new \PDO(
			'mysql:host=' . $config->get('host') . ';dbname=' . $config->get('db'),
			$config->get('user'),
			$config->get('password')
		);
		$pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

		$calendarBackend                                 = new \DPCalendar\Sabre\CalDAV\Backend\DPCalendar($pdo);
		$calendarBackend->calendarTableName              = $config->get('dbprefix') . 'dpcalendar_caldav_calendars';
		$calendarBackend->calendarObjectTableName        = $config->get('dbprefix') . 'dpcalendar_caldav_calendarobjects';
		$calendarBackend->calendarChangesTableName       = $config->get('dbprefix') . 'dpcalendar_caldav_calendarchanges';
		$calendarBackend->calendarInstancesTableName     = $config->get('dbprefix') . 'dpcalendar_caldav_calendarinstances';
		$calendarBackend->calendarSubscriptionsTableName = $config->get('dbprefix') . 'dpcalendar_caldav_calendarsubscriptions';

		return $calendarBackend;
	}
}
