<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2022 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Installer\InstallerScript;
use Joomla\Filesystem\Folder;

class PlgSystemDPCalendarrsformInstallerScript extends InstallerScript
{
	protected $minimumJoomla   = '4.1.0';
	protected $minComponent    = '3.1.0';
	protected $allowDowngrades = true;

	public function preflight($type, $parent)
	{
		if (!parent::preflight($type, $parent)) {
			return false;
		}

		if ($type == 'uninstall') {
			return true;
		}

		if (!file_exists(JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/rsform.php')) {
			throw new Exception('Please install the RSForm!Pro component before continuing.');
		}

		if (!file_exists(JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/assets.php')
			|| !file_exists(JPATH_ADMINISTRATOR . '/components/com_rsform/helpers/version.php')) {
			throw new Exception(sprintf('Please upgrade RSForm!Pro to at least version %s before continuing!', $this->minComponent));
		}

		// Check version matches
		require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/version.php';

		if (!class_exists('RSFormProVersion') || version_compare((string) new RSFormProVersion(), $this->minComponent, '<')) {
			throw new Exception(sprintf('Please upgrade RSForm!Pro to at least version %s before continuing!', $this->minComponent));
		}

		// Copy /admin files
		$src  = $parent->getParent()->getPath('source') . '/helper';
		$dest = JPATH_ADMINISTRATOR . '/components/com_rsform/helpers';

		if (!Folder::copy($src, $dest, '', true)) {
			throw new Exception('Could not copy to ' . str_replace(JPATH_ADMINISTRATOR, '', $dest) . ', please make sure destination is writable!');
		}

		// Update? Run our SQL file to ensure the fields are correctly set
		if ($type === 'update') {
			$sqlFile = $parent->getParent()->getPath('source') . '/sql/mysql/install.sql';
			if (!file_exists($sqlFile)) {
				return true;
			}

			$buffer = file_get_contents($sqlFile);
			if ($buffer === false) {
				return true;
			}

			$db      = Factory::getDbo();
			$queries = $db->splitSql($buffer);
			foreach ($queries as $query) {
				$query = trim($query);
				if ($query !== '') {
					$db->setQuery($query)->execute();
				}
			}
		}

		return true;
	}
}
