<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2022 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */

namespace DPCalendarPay\QR;

defined('_JEXEC') or die();

use DPCalendar\Helper\Booking;
use DPCalendar\Helper\DateHelper;
use DPCalendar\Helper\DPCalendarHelper;
use DPCalendar\Translator\Translator;
use Joomla\CMS\Factory;
use Joomla\CMS\Mail\Mail;
use Joomla\CMS\Plugin\PluginHelper;
use League\Pipeline\StageInterface;

class SendQRCodeInvoice implements StageInterface
{
	/**
	 * @var Mail
	 */
	private $mailer;

	public function __construct(Mail $mailer)
	{
		$this->mailer = $mailer;
	}

	public function __invoke($payload)
	{
		$payload->mailVariables['qrcode']          = $payload->qrcode;
		$payload->mailVariables['ibanNumber']      = $payload->ibanNumber;
		$payload->mailVariables['referenceNumber'] = $payload->referenceNumber;
		$payload->mailVariables['paymentDueDate']  = DPCalendarHelper::getDate($payload->item->book_date, true)->modify('+30 days')
			->format($payload->mailParams->get('event_date_format', 'd.m.Y'));

		$booking    = $payload->item;
		$tickets    = $payload->item->tickets;
		$translator = new Translator();
		$dateHelper = new DateHelper();
		$params     = $payload->mailParams;
		$variables  = $payload->mailVariables;
		ob_start();
		require PluginHelper::getLayoutPath('dpcalendarpay', 'qr', 'invoice');
		$payload->mailVariables['invoiceDetails'] = ob_get_clean();

		$subject = DPCalendarHelper::renderEvents(
			$payload->eventsWithTickets,
			$payload->language->_('PLG_DPCALENDARPAY_QR_PAYMENT_PROVIDER_TYPE_SWISSQR_INVOICE_SUBJECT_TEXT'),
			null,
			$payload->mailVariables
		);
		$body = DPCalendarHelper::renderEvents(
			$payload->eventsWithTickets,
			$payload->language->_('PLG_DPCALENDARPAY_QR_PAYMENT_PROVIDER_TYPE_SWISSQR_INVOICE_MESSAGE_TEXT'),
			null,
			$payload->mailVariables
		);

		if ($payload->mailParams->get('bookingsys_author_as_mail_from')) {
			foreach ($payload->eventsWithTickets as $event) {
				$this->mailer->setFrom(Factory::getUser($event->created_by)->email);
			}
		}

		$this->mailer->setSubject($subject);
		$this->mailer->setBody($body);
		$this->mailer->IsHTML(true);
		$this->mailer->addRecipient($payload->item->email);

		$fileName = Booking::createPDF($payload->mailVariables['invoiceDetails'], $payload->item->uid, true);
		if ($fileName) {
			$this->mailer->addAttachment($fileName);
			$files[] = $fileName;
		}

		try {
			$this->mailer->Send();
			foreach ($files as $file) {
				if (file_exists($file)) {
					unlink($file);
				}
			}
		} catch (\Exception $e) {
			foreach ($files as $file) {
				if (file_exists($file)) {
					unlink($file);
				}
			}

			throw $e;
		}

		return $payload;
	}
}
