<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2014 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
defined('_JEXEC') or die();

if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
	return;
}

class PlgDPCalendarPayStripe extends \DPCalendar\Plugin\PaymentPlugin
{
	protected function startTransaction($booking, $paymentProvider)
	{
		$params = $this->getPurchaseParameters($booking);

		$data = [
			'amount'               => str_replace('.', '', $params['amount']),
			'currency'             => $params['currency'],
			'payment_method_types' => ['card'],
			'receipt_email'        => $booking->email
		];

		$response = $this->getHTTP()->post(
			'https://api.stripe.com/v1/payment_intents',
			http_build_query($data),
			trim($paymentProvider->secret_key)
		);

		if (!empty($response->error)) {
			throw new Exception($response->error->message);
		}

		$booking->intent = $response->client_secret;

		return parent::startTransaction($booking, $paymentProvider);
	}

	protected function finishTransaction($data, $booking, $paymentProvider)
	{
		$response = $this->getHTTP()->get(
			'https://api.stripe.com/v1/payment_intents/' . $data['intent'],
			trim($paymentProvider->secret_key)
		);

		if (!empty($response->error)) {
			throw new Exception($response->error->message);
		}

		$updates = [
			'processor_key'  => $response->source,
			'transaction_id' => $response->id,
			'txn_type'       => $response->payment_method_types[0],
			'txn_currency'   => $response->currency,
			'raw_data'       => json_encode($response)
		];

		// Determine the state
		switch ($response->status) {
			case 'requires_payment_method':
			case 'requires_confirmation':
			case 'requires_action':
			case 'processing':
			case 'requires_capture':
				$updates['state'] = 4;
				break;
			case 'succeeded':
				$updates['state'] = 1;
				break;
			case 'canceled':
				$updates['state'] = 6;
				break;
		}

		$transaction = $this->getHTTP()->get(
			'https://api.stripe.com/v1/balance_transactions/' . $response->charges->data[0]->balance_transaction,
			trim($paymentProvider->secret_key)
		);

		if (!empty($transaction->error)) {
			throw new Exception($transaction->error->message);
		}

		// Get the fee
		if (is_numeric($transaction->fee)) {
			$updates['payment_fee'] = number_format($transaction->fee / 100, 2);
		}

		// Get the amount
		if (is_numeric($transaction->amount)) {
			$updates['gross_amount'] = number_format($transaction->amount / 100, 2);
		}

		// get the net amount
		if (is_numeric($transaction->net)) {
			$updates['net_amount'] = number_format($transaction->net / 100, 2);
		}

		return $updates;
	}
}
