<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2022 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */

defined('_JEXEC') or die();

use DPCalendar\Helper\FieldsOrder;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;

$user = Factory::getUser($booking->user_id);

$eventOptions   = [];
$orderedOptions = $booking->options ? explode(',', $booking->options) : [];
foreach ($tickets as $ticket) {
	if (!$ticket->event_options) {
		continue;
	}

	$ticketEventOptions = $ticket->event_options;
	if (is_string($ticketEventOptions)) {
		$ticketEventOptions = json_decode($ticketEventOptions);
	}

	// Prepare the options
	foreach ($ticketEventOptions as $key => $option) {
		$key = preg_replace('/\D/', '', $key);

		foreach ($orderedOptions as $o) {
			list($eventId, $type, $amount) = explode('-', $o);

			if ($eventId != $ticket->event_id || $type != $key || !empty($eventOptions[$eventId][$type])) {
				continue;
			}

			if (!array_key_exists($eventId, $eventOptions)) {
				$eventOptions[$eventId] = [];
			}
			$eventOptions[$eventId][$type] = ['price' => $option->price, 'label' => $option->label, 'amount' => $amount];
		}
	}
}

$imageUrl = $params->get('pdf_logo', '');
if ($imageUrl && !filter_var($imageUrl, FILTER_VALIDATE_URL)) {
	$imageUrl = trim(Uri::root(), '/') . '/' . trim($imageUrl, '/');
}

if ($pos = strpos($imageUrl, '#')) {
	$imageUrl = substr($imageUrl, 0, $pos);
}

$totalPrice = 0;

$fields = FieldsOrder::getBookingFields($booking, $params, Factory::getApplication());
?>
<html>
	<head>
		<style>
			@page {
				margin: 6rem 4rem;
			}
			.dp-body {
				font-size: 8;
			}
			.dp-header {
				position: fixed;
				top: -3rem;
				left: 0px;
				right: 0px;
				height: 1.5rem;
				border-bottom: 1px solid #000;
			}
			.dp-title {
				margin-top: 2rem;
				margin-bottom: 0.5rem;
				font-weight: bold;
			}
			.dp-description__label {
				display: inline-block;
				width: 180px;
			}
			.dp-description__content {
				display: inline-block;
			}
			.dp-information__logo, .dp-information__address {
				display: inline-block;
				width: 49%;
				vertical-align: top;
			}
			.dp-information__logo img {
				max-width: 100%;
				max-height: 200px;
			}
			.dp-table {
				width: 100%;
				margin: 2rem 0;
				font-size: 8;
			}
			.dp-table th, .dp-table td {
				text-align: left;
			}
			.dp-table th {
				padding-top: 1rem;
			}
			.dp-table .dp-table__price-cell {
				text-align: right;
			}
			.dp-table__sum-row td {
				padding: 1rem 0;
				border-top: 1px solid #eee;
				font-weight: bold;
			}
			#qr-bill {
				width: 100%;
				page-break-inside: avoid;
			}
			#qr-bill-amount {
				float: right;
			}
			#qr-bill-payment-part p {
				font-size: 8pt !important;
				line-height: 9pt !important;
			}
		</style>
	</head>
	<body class="dp-body">
		<header class="dp-header"><?php echo $params->get('pdf_header'); ?></header>
		<main class="dp-booking-invoice">
			<?php if ($content = $params->get('pdf_content_top', '')) { ?>
				<p><?php echo nl2br($translator->translate($content)); ?></p>
			<?php } ?>
			<div class="dp-information">
				<div class="dp-information__logo">
					<?php if ($imageUrl) { ?>
						<img src="<?php echo $imageUrl; ?>"/>
					<?php } ?>
					<div class="dp-information__company-address">
						<?php echo nl2br($params->get('pdf_address', '')); ?>
					</div>
				</div>
				<div class="dp-information__address">
					<?php foreach ($fields as $field) { ?>
						<div><?php echo $translator->translate($field->dpDisplayContent); ?></div>
					<?php } ?>
				</div>
			</div>
			<div class="dp-title">
				<?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_INVOICE_DETAILS'); ?>
			</div>
			<div class="dp-description">
				<div class="dp-description__label">
					<?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_INVOICE_NUMBER'); ?>
				</div>
				<div class="dp-description__content">
					<?php echo $booking->uid; ?>
				</div>
			</div>
			<div class="dp-description">
				<div class="dp-description__label">
					<?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_INVOICE_DATE'); ?>
				</div>
				<div class="dp-description__content">
					<div class="dp-description__content">
						<?php echo $dateHelper->getDate($booking->book_date)->format($params->get('event_date_format', 'd.m.Y'), true); ?>
					</div>
				</div>
			</div>
			<div class="dp-description">
				<div class="dp-description__label">
					<?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_PAYMENT_DATE'); ?>
				</div>
				<div class="dp-description__content">
					<div class="dp-description__content"><?php echo $variables['paymentDueDate']; ?></div>
				</div>
			</div>
			<table class="dp-table">
				<tr>
					<th><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_EVENT'); ?></th>
					<th><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_TICKET_TYPE'); ?></th>
					<th class="dp-table__price-cell"><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_PRICE'); ?></th>
				</tr>
				<?php foreach ($tickets as $ticket) { ?>
					<?php $totalPrice += $ticket->price; ?>
					<tr>
						<td>
							<?php echo $ticket->event_title; ?> [
							<?php echo $dateHelper->getDateStringFromEvent(
								$ticket,
								$params->get('event_date_format'),
								$params->get('event_time_format')
							); ?>
							]
						</td>
						<td>
							<?php if (!empty($ticket->event_prices)) { ?>
								<?php $prices = !is_string($ticket->event_prices) ? $ticket->event_prices : json_decode($ticket->event_prices); ?>
								<?php if (!empty($prices->label[$ticket->type])) { ?>
									<?php echo $prices->label[$ticket->type]; ?>
								<?php } ?>
							<?php } ?>
						</td>
						<td class="dp-table__price-cell">
							<?php echo DPCalendarHelper::renderPrice($ticket->price, $params->get('currency_symbol', '$')); ?>
						</td>
					</tr>
				<?php } ?>
				<?php if (!empty($eventOptions)) { ?>
					<tr>
						<th><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_OPTION'); ?></th>
						<th><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_AMOUNT'); ?></th>
						<th><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_PRICE'); ?></th>
					</tr>
					<?php foreach ($eventOptions as $eventId => $options) { ?>
						<?php foreach ($options as $option) { ?>
							<?php $totalPrice += $option['price']; ?>
							<tr>
								<td><?php echo $option['label']; ?></td>
								<td><?php echo $option['amount']; ?></td>
								<td class="dp-table__price-cell"><?php echo DPCalendarHelper::renderPrice($option['price']); ?></td>
							</tr>
						<?php } ?>
					<?php } ?>
				<?php } ?>
				<?php if ($totalPrice != $booking->price) { ?>
					<tr class="dp-table__sum-row">
						<td colspan="2"><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_TOTAL'); ?></td>
						<td class="dp-table__price-cell">
							<?php echo DPCalendarHelper::renderPrice($totalPrice, $params->get('currency_symbol', '$')); ?>
						</td>
					</tr>
				<?php } ?>
				<?php if ((float)$booking->coupon_rate) { ?>
					<tr>
						<td colspan="2"><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_COUPON_RATE'); ?></td>
						<td class="dp-table__price-cell">
							- <?php echo DPCalendarHelper::renderPrice($booking->coupon_rate, $params->get('currency_symbol', '$')); ?>
						</td>
					</tr>
				<?php } ?>
				<?php if ($booking->tax && $booking->tax != '0.00') { ?>
					<tr>
						<td colspan="2"><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_TAX'); ?></td>
						<td class="dp-table__price-cell">
							<?php echo DPCalendarHelper::renderPrice($booking->tax, $params->get('currency_symbol', '$')); ?>
						</td>
					</tr>
				<?php } ?>
				<tr class="dp-table__sum-row">
					<td colspan="2"><?php echo $translator->translate('PLG_DPCALENDARPAY_QR_INVOICE_TOTAL'); ?></td>
					<td class="dp-table__price-cell">
						<?php echo DPCalendarHelper::renderPrice($booking->price, $params->get('currency_symbol', '$')); ?>
					</td>
				</tr>
			</table>
			<div><?php echo $variables['qrcode']; ?></div>
			<?php if ($content = $params->get('pdf_content_bottom')) { ?>
				<p><?php echo nl2br($translator->translate($content)); ?></p>
			<?php } ?>
		</main>
	</body>
</html>
