<?php
/**
 * @package   DPCalendar
 * @copyright Copyright (C) 2014 Digital Peak GmbH. <https://www.digital-peak.com>
 * @license   http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 */
defined('_JEXEC') or die();

if (!JLoader::import('components.com_dpcalendar.helpers.dpcalendar', JPATH_ADMINISTRATOR)) {
	return;
}

use DigitalPeak\ThinHTTP as HTTP;
use Joomla\CMS\Date\Date;
use Joomla\Registry\Registry;

class PlgDPCalendarTwitter extends \DPCalendar\Plugin\SyncPlugin
{
	protected $identifier = 'tw';

	protected $app;

	protected function getSyncToken($calendar)
	{
		$queryParams = ['count' => 1];
		if ($calendar->sync_token > 1) {
			$queryParams['since_id'] = $calendar->sync_token;
		}

		try {
			$data = $this->callAPI('user_timeline', $calendar->params, $queryParams);
			if (!empty($data->errors)) {
				$this->log($data->errors[0]->message);

				return $calendar->sync_token;
			}

			foreach ($data->data as $tweet) {
				if (empty($tweet->id)) {
					continue;
				}

				return $tweet->id;
			}
		} catch (Exception $e) {
			$this->log($e->getMessage());
		}

		return $calendar->sync_token;
	}

	protected function getContent($calendarId, Date $startDate = null, Date $endDate = null, Registry $options)
	{
		$calendar = $this->getDbCal($calendarId);
		if (empty($calendar)) {
			return '';
		}

		$params = $calendar->params;

		$cursor = -1;
		$text   = [];
		$text[] = 'BEGIN:VCALENDAR';
		while ($cursor > 0 || $cursor < 0) {
			try {
				$data = $this->callAPI('user_timeline', $params, ['count' => 200, 'cursor' => $cursor]);
				if (!empty($data->errors)) {
					$this->log($data->errors[0]->message);

					return '';
				}
			} catch (Exception $e) {
				$this->log($e->getMessage());

				return '';
			}

			$cursor = isset($data->next_cursor_str) ? $data->next_cursor_str : 0;
			foreach ($data->data as $tweet) {
				if (!is_object($tweet)) {
					continue;
				}
				$text[] = 'BEGIN:VEVENT';

				$start  = DPCalendarHelper::getDate($tweet->created_at);
				$text[] = 'DTSTART:' . $start->format('Ymd\THis\Z');
				$start->modify('+1 minute');
				$text[] = 'DTEND:' . $start->format('Ymd\THis\Z');

				$text[] = 'UID:' . md5($tweet->id . 'Twitter');
				$text[] = 'SUMMARY:' . $tweet->text;
				$text[] = 'DESCRIPTION:' . DPCalendarHelper::parseHtml($tweet->text);
				$text[] = 'LAST-MODIFIED:' . $start->format('Ymd\THis\Z');
				$text[] = 'CREATED:' . $start->format('Ymd\THis\Z');

				if (isset($tweet->place)) {
					$text[] = 'LOCATION:' . $tweet->place->full_name;
				}

				$text[] = 'X-URL:http://twitter.com/' . $tweet->user->screen_name . '/status/' . $tweet->id_str;

				$text[] = 'END:VEVENT';
			}
		}
		$text[] = 'END:VCALENDAR';

		return $text;
	}

	public function onContentBeforeDisplay($context, $event)
	{
		if ($context != 'com_dpcalendar.event') {
			return;
		}

		$doc = $this->app->getDocument();
		$doc->addCustomTag('<meta name="twitter:title" content="' . htmlspecialchars($event->title) . '"/>');
		$doc->addCustomTag('<meta name="twitter:description" content="' . strip_tags($event->description) . '"/>');
		$doc->addCustomTag('<meta name="twitter:url" content="' . str_replace('"', '&quot;', JURI::current()) . '"/>');

		if (!empty($event->images->image_full)) {
			$doc->addCustomTag('<meta name="twitter:image" content="' . htmlspecialchars(JURI::root() . $event->images->image_full) . '"/>');
		}

		if (!empty($event->images->image_full_alt)) {
			$doc->addCustomTag('<meta name="twitter:image:alt" content="' . htmlspecialchars($event->images->image_full_alt) . '"/>');
		}

		foreach ($this->fetchCalendars() as $calendar) {
			$handle = htmlspecialchars(str_replace(' ', '', $calendar->title));
			$doc->addCustomTag('<meta name="twitter:site" content="@' . $handle . '"/>');
			$doc->addCustomTag('<meta name="twitter:creator" content="@' . $handle . '"/>');
			break;
		}
	}

	public function onContentDisplayEventList($context, $calendarIds, $events)
	{
		if ($context != 'com_dpcalendar.calendar' || !$calendarIds) {
			return;
		}

		$doc = $this->app->getDocument();
		foreach ($calendarIds as $calendarId) {
			$calendar = \DPCalendar\Helper\DPCalendarHelper::getCalendar($calendarId);
			$doc->addCustomTag('<meta name="twitter:card" content="summary_large_image" />');
			$doc->addCustomTag('<meta name="twitter:title" content="' . htmlspecialchars($calendar->title) . '"/>');
			$doc->addCustomTag('<meta name="twitter:description" content="' . strip_tags($calendar->description) . '"/>');
			$doc->addCustomTag('<meta name="twitter:url" content="' . str_replace('"', '&quot;', JURI::current()) . '"/>');

			if (!empty($calendar->params->get('image'))) {
				$doc->addCustomTag('<meta name="twitter:image" content="' . htmlspecialchars(JURI::root() . $calendar->params->get('image')) . '"/>');
			}
			if (!empty($calendar->params->get('image_alt'))) {
				$doc->addCustomTag('<meta name="twitter:image:alt" content="' . htmlspecialchars(JURI::root() . $calendar->params->get('image_alt')) . '"/>');
			}

			if (!empty($calendar->params->get('image')) || !$events) {
				break;
			}

			$event = reset($events);
			if (!empty($event->images->image_full)) {
				$doc->addCustomTag('<meta name="twitter:image" content="' . htmlspecialchars(JURI::root() . $event->images->image_full) . '"/>');
			}
			if (!empty($event->images->image_full_alt)) {
				$doc->addCustomTag('<meta name="twitter:image:alt" content="' . htmlspecialchars($event->images->image_full_alt) . '"/>');
			}
			break;
		}

		foreach ($this->fetchCalendars() as $calendar) {
			$handle = htmlspecialchars(str_replace(' ', '', $calendar->title));
			$doc->addCustomTag('<meta name="twitter:site" content="@' . $handle . '"/>');
			$doc->addCustomTag('<meta name="twitter:creator" content="@' . $handle . '"/>');
			break;
		}
	}

	public function onContentAfterSave($context, $item, $isNew)
	{
		if ($context != 'com_dpcalendar.event') {
			return true;
		}

		foreach ($this->fetchCalendars() as $calendar) {
			if ($isNew && $calendar->params->get('creationeventupdate', 1)) {
				$this->autoTweetEvent('create', $item, $calendar);
			}

			if (!$isNew && $calendar->params->get('updateeventupdate', 1)) {
				$this->autoTweetEvent('save', $item, $calendar);
			}
		}
	}

	public function onContentAfterDelete($context, $item)
	{
		if ($context != 'com_dpcalendar.event') {
			return true;
		}

		foreach ($this->fetchCalendars() as $calendar) {
			if ($calendar->params->get('deletioneventupdate', 0)) {
				$this->autoTweetEvent('delete', $item, $calendar);
			}
		}
	}

	private function autoTweetEvent($action, $event, $calendar)
	{
		// We don't tweet external events
		if (!is_numeric($event->catid)) {
			return;
		}

		$params = $calendar->params;
		$ids    = $params->get('ids');
		if (!$ids) {
			return;
		}

		if ($params->get('eventsinsubcat', 0)) {
			$sids = $ids;
			foreach ($sids as $id) {
				$cat = JCategories::getInstance('DPCalendar')->get($id);
				if (!$cat) {
					continue;
				}

				foreach ($cat->getChildren() as $ch) {
					$ids[] = $ch->id;
				}
			}
		}

		if (!in_array($event->catid, $ids)) {
			return;
		}

		$status   = '';
		$eventurl = DPCalendarHelperRoute::getEventRoute($event->id, $event->catid, true);
		$eventurl = str_replace('administrator/', '', $eventurl);
		switch ($action) {
			case 'create':
				$status = JText::sprintf('PLG_DPCALENDAR_TWITTER_STATUS_EVENT_CREATED', $event->title, $eventurl);
				break;
			case 'save':
				$status = JText::sprintf('PLG_DPCALENDAR_TWITTER_STATUS_EVENT_UPDATED', $event->title, $eventurl);
				break;
			case 'delete':
				$status = JText::sprintf('PLG_DPCALENDAR_TWITTER_STATUS_EVENT_DELETED', $event->title);
				break;
		}
		$status = substr($status, 0, 280);

		try {
			$response = $this->callAPI('update', $params, ['status' => $status], 'POST');
			if (!empty($response->errors)) {
				throw new Exception($response->errors[0]->message);
			}

			$tweetUrl = 'https://twitter.com/' . $response->user->screen_name . '/status/' . $response->id_str;
			$this->app->enqueueMessage(
				JText::sprintf('PLG_DPCALENDAR_TWITTER_PUBLISH_SUCCESS', $tweetUrl, $calendar->title),
				'success'
			);
		} catch (Exception $e) {
			$this->log(JText::sprintf('PLG_DPCALENDAR_TWITTER_PUBLISH_ERROR', $e->getMessage()));
		}
	}

	private function callAPI($action, \Joomla\Registry\Registry $params, $queryParams, $method = 'GET')
	{
		$url = 'https://api.twitter.com/1.1/statuses/' . $action . '.json';

		$oauth = [
			'oauth_consumer_key'     => $params->get('apikey'),
			'oauth_nonce'            => time(),
			'oauth_signature_method' => 'HMAC-SHA1',
			'oauth_timestamp'        => time(),
			'oauth_token'            => $params->get('accesstoken'),
			'oauth_version'          => '1.0'
		];
		$oauth  = $method == 'GET' ? $queryParams + $oauth : $oauth + $queryParams;
		$values = [];
		foreach ($oauth as $key => $value) {
			$values[] = $key . '="' . rawurlencode($value) . '"';
		}

		$compositeKey = $params->get('apisecret') . '&' . $params->get('accesstokensecret');
		$data         = $method;
		$data .= '&' . rawurlencode($url);
		$data .= '&' . rawurlencode(str_replace('"', '', implode('&', $values)));
		$values[] = 'oauth_signature="' . rawurlencode(base64_encode(hash_hmac('sha1', $data, $compositeKey, true))) . '"';

		if ($method == 'POST') {
			return (new HTTP())->post(
				$url,
				http_build_query($queryParams),
				null,
				null,
				['Authorization: OAuth ' . implode(', ', $values)]
			);
		}

		return (new HTTP())->get(
			$url . '?' . http_build_query($queryParams),
			null,
			null,
			['Authorization: OAuth ' . implode(', ', $values)]
		);
	}
}
